/*******************************************************************
  Components for embedded applications builded for
  laboratory and medical instruments firmware  (COLAMI)
 
  drvsupport_mx1.c - generic support routines for drivers writing
 
  Copyright holders and project originators
    (C) 2001-2004 by Pavel Pisa pisa@cmp.felk.cvut.cz
    (C) 2002-2004 by PiKRON Ltd. http://www.pikron.com

 The COLAMI components can be used and copied under next licenses
   - MPL - Mozilla Public License
   - GPL - GNU Public License
   - LGPL - Lesser GNU Public License
   - and other licenses added by project originators
 Code can be modified and re-distributed under any combination
 of the above listed licenses. If contributor does not agree with
 some of the licenses, he can delete appropriate line.
 Warning, if you delete all lines, you are not allowed to
 distribute code or build project.
 *******************************************************************/

#include <rtems.h>
#include <rtems/io.h>
#include <rtems/libio.h>
#include <rtems/error.h>
#include <malloc.h>
#include <string.h>
#include <mc9328mxl.h>
#include <mx1_reg.h>
#include <irq.h>
#include <drvsupport.h>

/* Enables spi_mx1 interrupts. */
void 
drvsup_mx1_isr_on(const rtems_irq_connect_data *isr_data)
{
	/* Enable interrupts */
	MC9328MXL_AITC_INTENNUM = isr_data->name;

	return;
}

/* Disables spi_mx1 interrupts */
void
drvsup_mx1_isr_off(const rtems_irq_connect_data *isr_data)
{
	/* disable all various TX/RX interrupts */
	MC9328MXL_AITC_INTDISNUM = isr_data->name;

	return;
}

/* Tests to see if spi_mx1 interrupts are enabled, and returns non-0 if so.
 * If interrupt is not enabled, returns 0.
 */
int
drvsup_mx1_isr_is_on(const rtems_irq_connect_data *isr_data)
{
	if(isr_data->name<32)
		return MC9328MXL_AITC_INTENABLEL & (1 << ((isr_data->name)&0x1f));
	else
		return MC9328MXL_AITC_INTENABLEH & (1 << ((isr_data->name - 32)&0x1f));
}

/* Set new interrupt handler simple and clear way.
 */
int
drvsup_request_irq(rtems_irq_number irq_num, rtems_irq_hdl irq_handler,
			unsigned options, char *name, rtems_irq_hdl_param data)
{
	rtems_irq_connect_data *irq_connect_data;

	if(!irq_handler)
		return -1;

	irq_connect_data=malloc(sizeof(rtems_irq_connect_data));
	if(!irq_connect_data)
		return -1;

	memset(irq_connect_data, 0, sizeof(rtems_irq_connect_data));

	irq_connect_data->name	= irq_num;
	irq_connect_data->hdl	= irq_handler;
	irq_connect_data->handle= data;
	irq_connect_data->on	= drvsup_mx1_isr_on;
	irq_connect_data->off	= drvsup_mx1_isr_off;
	irq_connect_data->isOn	= drvsup_mx1_isr_is_on;
	#if 0
	irq_connect_data->irqLevel   = 3;	/* unused for ARM */
	irq_connect_data->irqTrigger = 0;	/* unused for ARM */
	#endif

	if (!BSP_install_rtems_irq_handler(irq_connect_data)){
		free(irq_connect_data);
		return -1;
	}

	return 0;
}

