/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Copyright (C) 2002 Motorola Semiconductors HK Ltd
 *
 */


/******************************************************************************
 *
 * Copyright (C) 2001, Motorola All Rights Reserved
 *
 * File Name:   mmcsd.c
 *
 * Progammers:  Yiwei Zhao
 *
 * Date of Creations:   30  DEC,2001
 *
 * Synopsis:
 *
 * Modification History:
 * 30 DEC, 2001
 *
 *****************************************************************************/


#include <system_def.h>
#include <mx1_reg.h>
#include "mmcsd_def.h"
#include "mmcsd_main.h"

/* Functions List:
 *
 */

unsigned long* pI2CReadData;
unsigned short nI2CReadData;//number of data
unsigned short nI2CData; // index of data
unsigned long  dI2CReadData[2];
unsigned char  bTransferDone;
unsigned char  bTransferCurMode;
unsigned char  gSDWriteProtect;

/*
 * Implementation of the MMCSD Driver.
 */
/******************************************************************************
 * Function Name: _MMCSD_InitCard
 *
 * Input:               None.
 *
 * Value Returned:
 *
 * 		MMCSD_STATUS
 *
 * Description:
 *
 * 	this routine will initialize the cards.
 *
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *****************************************************************************/

MMCSD_STATUS _MMCSD_InitSDHC(const MMCSD_Dev *device)
{
	MMCSD_STATUS	ret;

#ifdef CONFIG_ARCH_MX2ADS
	if(device->port == 0)//sdhc1
	{
		//set i/o driving strength
		_reg_SYS_DSCR1 &= ~0x07;
		//		_reg_SYS_DSCR1 |= 0x01;//4mA
		_reg_SYS_DSCR1 |= 0x03;//6mA

	}
	else if(device->port == 1)
	{
		_reg_SYS_DSCR1 &= ~0x00380000;
		//		_reg_SYS_DSCR1 |= 0x00080000;//4mA
		_reg_SYS_DSCR1 |= 0x00180000;//6mA
	}
#endif
	//check hardware revision
	ret=_MMCSD_ChectHardwareRevision(device);
	if(ret)
		return ret;
	//softreset
	ret=_MMCSD_SoftReset(device);
	if(ret)
		return ret;
	_MMCSD_SetClk(device, MMCSD_LOW_CLKRATE);
	_MMCSD_ReadTimeOut(device, 0x2DB4);

	ret=_MMCSD_ChectHardwareRevision(device);
	if(ret)
		return ret;
	ret=_MMCSD_SoftReset(device);
	if(ret)
		return ret;

	_MMCSD_SetClk(device,MMCSD_LOW_CLKRATE);

	_MMCSD_ReadTimeOut(device,0x2DB4);
	return 0;

}

MMCSD_STATUS _MMCSD_InitMMC(MMCSD_Dev *device)
{
	uint16_t  i,  count;
	MMCSD_STATUS  ret;
	uint8_t   Cid_buff[20];
	count = 0;
	ret = 0;

	//TRACE("Card Identify\n");
	/*------------------------------------------------------------------*/
	/*  CMD0 CALL PROCEDURE (PLACES CARDS IN IDLE STATE)                */
	/*------------------------------------------------------------------*/

	ret =_MMCSD_GoIdleState(device);
	if(ret != 0 )
	{	/*--------- WHEN NO ERROR OCCURS (= FINISH INIT)*/
		FAILED("Place Cards into Idle State Failed(%ld)\n",(unsigned long)ret);
		return ret;
	}

	/*------------------------------------------------------------------*/
	/*  CMD1						                 */
	/*------------------------------------------------------------------*/
	i = MMCSD_ADP_REPEATNUM;

	while (i--)
	{
		ret = _MMCSD_SendOpCond(device, MMCSD_MMC_CARD_VOLTAGE);
		if (ret == 0)
			break;  // card detected

		if (ret != MMCSD_CARD_BUSY)
		{
			//FAILED("Command SendOpCond Failed\n");
			return ret;		//CMD send ERROR
		}
	}
	/*------------------------------------------------------------------*/
	/* Register CARDS					                */
	/*------------------------------------------------------------------*/

	/*
	   TRACE("Set Op Condition..\n");
	   ret = _MMCSD_Set_SendOpCond(MMCSD_MMC_CARD_VOLTAGE,
	   MMCSD_CARD_SET_VOLTAGE);
	   if(ret)
	   {
	   FAILED("MMCSD_SendOpCond(SET_VOLTAGE) error with 0x%x\n",ret);
	   return ret;
	   }
	 */

	/*
	 * Aleph: only one card supported.
	 */
	ret = _MMCSD_AllSendCID(device, Cid_buff);  //CMD2
	if (ret & MMCSD_CMD_TIMEOUT)
	{
		FAILED("_MMCSD_AllSendCID failed with 0x%lx\n",(unsigned long)ret);
		device->card_state = MMCSD_NOTREADY;
		return ret;
	}

	ret = _MMCSD_SetRelativeAddr(device, &device->card_rca); //CMD3

	device->card_state = MMCSD_STANDBY;
	count++;

	/*----------------------------------------------------------------------*/
	/*  CHECK IF EFFECTIVE CARDS EXIST                                      */
	/*----------------------------------------------------------------------*/
	if ( count )
	{	/*------------------------- IF THERE IS AN EFFECTIVE CARD  */
		g_mmcsd_num = count;                  /* ACQUIRE THE NUMBER  */
		//TRACE("Finish Init Card. %d Cards have beed detected\n",count);
		return MMCSD_OK;
	}
	else
	{      /*--------------------- WHEN NO EFFECTIVE CARD IS IDENTIFIED */
		INFO("No Card is detected\n");
		return( MMCSD_NO_CARD );
	}
	return( ret );
}

/******************************************************************************
 * Function Name: _MMCSD_InitCard
 *
 * Input:
 * 		device:	 pointer to MMCSD_Dev structure.
 *
 * Value Returned:
 *
 * 		MMCSD_STATUS
 *
 * Description:
 *
 *	Check and aqquire CSD/CID register data
 *
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *****************************************************************************/

MMCSD_STATUS _MMCSD_GetInfo(MMCSD_Dev *device)
{
	MMCSD_STATUS  ret;
	uint8_t   Csd_buff[20];                /* BUFFER FOR RECEIVING CSD           */
	uint8_t   Cid_buff[20];                /* BUFFER FOR RECEIVING CID           */
	TRACE("Get Card Info\n");
	ret = 0;
	_MMCSD_SetBlockLenReg(device, 0x10);		// Why????

	ret = _MMCSD_SendCSD(device,  device->card_rca,Csd_buff); //CMD9
	if (ret)
	{
		FAILED("_MMCSD_SendCSD() error with 0x%lx\n",(unsigned long)ret);
		return ret;
	}
	else
	{
		ret = _MMCSD_CSDCheck(Csd_buff);
		if ( ret )
		{
			FAILED("_MMCSD_CSDCheck error, ret=0x%x\n",ret);
			return ret;
		}
		else
		{
			_MMCSD_SetCSD(device,Csd_buff);
		}
	}

	ret = _MMCSD_SendCID(device,  device->card_rca,Cid_buff); //CMD10
	if ( ret )
	{
		FAILED("_MMCSD_SendCID() error with 0x%lx\n",(unsigned long)ret);
		device->card_state = MMCSD_NOTREADY;
	}
	else
	{
		ret = _MMCSD_CIDCheck(Cid_buff);
		if ( ret )
		{
			FAILED("_MMCSD_CIDCheck() error with 0x%x\n",ret);
			return ret;
		}
		else
		{
			_MMCSD_SetCID(device, Cid_buff);
		}
	}
	return( ret );
}

/******************************************************************************
 * Function Name: _MMCSD_SetSectLen
 *
 * Input:               None.
 *
 * Value Returned:
 *
 * 		MMCSD_STATUS
 *
 * Description:
 *
 *
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *      Why??? this function must be here, otherwise CRC-WRITE-ERROR while polling write.
 *****************************************************************************/

MMCSD_STATUS _MMCSD_SetSectLen(MMCSD_Dev *device)
{

	uint8_t   type;
	MMCSD_STATUS  ret;
	/*----------------------------------------------------------------------*/
	/*  INITIALIZE LOCAL VARIABLES                                          */
	/*----------------------------------------------------------------------*/
	type     = 0;                         /* CMD TYPE = NO DATA TRANSFER    */
	ret      = 0;                                   /* SEND/RECEIVE ERROR   */

	/*--------------------------------------------------------------*/
	/*  CARD SELECT                                                 */
	/*--------------------------------------------------------------*/

	ret = _MMCSD_SelectDeselectCard(device,device->card_rca);
	if ( ret )
	{
		device->card_state = MMCSD_NOTREADY;
		return ret;
	}
	/*
	   ret = _MMCSD_SetBlockLen(MMCSD_MINIMUM_BLK_SIZE,1); //
	   if ( ret )
	   {
	   device->card_state = MMCSD_NOTREADY;
	   return ret;
	   }
	   device->blksize = MMCSD_MINIMUM_BLK_SIZE;
	 */
	ret = _MMCSD_SelectDeselectCard(device,0);
	if ( ret )
	{
		device->card_state = MMCSD_NOTREADY;
		return ret;
	}
	device->card_state = MMCSD_STANDBY;

	return( ret );
}
/******************************************************************************
 * Function Name: _MMCSD_GetStatus
 *
 * Input:               None.
 *
 * Value Returned:
 *
 * 		MMCSD_STATUS
 *
 * Description:
 *
 *
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *****************************************************************************/

uint32_t _MMCSD_GetStatus(MMCSD_Dev *device)
{
	uint32_t state;
	MMCSD_STATUS ret=_MMCSD_SendStatus(device,device->card_rca,&state);

	ret&=MMCSD_STATUS_NOERROR|0xFFFF;

	if ( ret )
	{
		device->card_state = MMCSD_NOTREADY;
	}
	else
	{
		device->card_state = MMCSD_STANDBY;
	}
	return( ret );
}

static const unsigned int tran_exp[] = {
	10000,		100000,		1000000,	10000000,
	0,		0,		0,		0
};

static const unsigned char tran_mant[] = {
	0,	10,	12,	13,	15,	20,	25,	30,
	35,	40,	45,	50,	55,	60,	70,	80,
};

/******************************************************************************
 * Function Name: _MMCSD_SetCSD
 *
 * Input:               None.
 *
 * Value Returned:
 *
 * Description:
 *
 * 	Get CSD Info
 *
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *****************************************************************************/
void _MMCSD_SetCSD(MMCSD_Dev *device,uint8_t * Csd_buff)
{
	uint32_t size,t1,t2;
	uint32_t read_bl_len;         /* read data block length     */
	uint32_t c_size;              /* device size                */
	uint32_t mult,blocknr,block_len;
	uint32_t c_size_mult;
	uint8_t write_blk_len;
	uint8_t erase_sector_size;         /* erase  sector size          */
	uint8_t wp_grp_size;

	read_bl_len=( Csd_buff[5] & 0x0F );
	//TRACE("Read Block Length [5] 0x%x\n",read_bl_len);

	device->ccc = (Csd_buff[5]&0xF0)>>4;
	device->ccc |= Csd_buff[4]<<4;

	/* maximal data transfer rate, 3 bits at 96 is exp., 4 bits at 99 are mant. */
	device->max_dtr = tran_mant[(Csd_buff[3]>>3)&0xf];
	device->max_dtr *= tran_exp[Csd_buff[3]&0x7];

	c_size=0;
	t1=Csd_buff[6]&0x3;
	t1=t1<<10;

	t2=Csd_buff[7];
	t2=t2<<2;
	t2=t1|t2;

	c_size=(t2|((Csd_buff[8]&0xC0)>>6));

	//erase_sector_size=( ( Csd_buff[10] & 0x7C ) >> 2 );
	erase_sector_size=0;
	erase_sector_size=((Csd_buff[10]&0x3F)<<1);
	erase_sector_size= erase_sector_size|((Csd_buff[11]&0x80)>>7);
	erase_sector_size++;

	wp_grp_size = 0;
	wp_grp_size = (Csd_buff[11]&0x3F)+1;


	write_blk_len = 0;
	write_blk_len = (Csd_buff[12]&0x03)<<2;
	write_blk_len = write_blk_len|((Csd_buff[13]&0xc0)>>6);

	c_size_mult=( ( ( Csd_buff[9]  & 0x03 ) << 1 )|
			( ( Csd_buff[10] & 0x80 ) >> 7 ) );

	mult=1<<(c_size_mult+2);
	blocknr=(c_size+1)*mult;
	block_len=1<<read_bl_len;

	size=blocknr*block_len;

	device->dev_size=size;
	device->sect_num=size/MMCSD_MINIMUM_BLK_SIZE;
	device->blksize = MMCSD_MINIMUM_BLK_SIZE;
	device->blksize_bit = 9;

	device->phy_blksize = 1<<write_blk_len;
	device->phy_sectorsize = (device->phy_blksize)*erase_sector_size;
	device->wp_grp_size = (device->phy_sectorsize)*wp_grp_size;

	INFO("write block size is  %d",device->phy_blksize);
	INFO("Erase sector size is %ld",(long int)(device->phy_sectorsize));
	INFO("Card Size is %ld bytes,  sect_num (for FAT file system) is %ld\n",
			(long int)size,(long int)size/MMCSD_MINIMUM_BLK_SIZE);
	INFO("Supported Card Command Class is 0x%0X\n",device->ccc);
	//INFO("Wirte Protected group size [11] = 0x%x",wp_grp_size);
	INFO("Maximal Data Transfer Rate is %d.%06d MHz\n",
		device->max_dtr/1000000,device->max_dtr%1000000);

}
/******************************************************************************
 * Function Name: _MMCSD_SetCID
 *
 * Input:               None.
 *
 * Value Returned:
 *
 * Description:
 *
 *	Set CID
 *
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *****************************************************************************/
void _MMCSD_SetCID(MMCSD_Dev *device,uint8_t* Cid_buff)
{

	int j;
	for(j=0;j<16;j++)
		device->CID[j]=Cid_buff[j];
}


/******************************************************************************
 * Function Name: _MMCSD_CSDCheck
 *
 * Input:               None.
 *
 * Value Returned:
 *
 * 		MMCSD_STATUS
 *
 * Description:
 *
 * 	Check CSD register
 *
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *****************************************************************************/

/****************************************************************************
  CSD Register
  -----------------------------------------------------------------------------
  Name                        | Width| CSD_Slice|    pCsdbuff (BYTE)
  -----------------------------------------------------------------------------
  CSD struction				   2     127:126
  System spec. version           4     125:122           0
  reserved                       2     121:120
  -----------------------------------------------------------------------------
  data read access time 1        8     119:112           1
  -----------------------------------------------------------------------------
  data read access time 2        8     111:104           2
  -----------------------------------------------------------------------------
  Max data transfer rate         8     103:96            3
  -----------------------------------------------------------------------------
  card command classes           12     95:84            4
  max read data block length     4      83:80            5
  ------------------------------------------------------------------------------
  partial blocks for read length  1     79:79
  wirte block misalignment        1     78:78
  read block misalignment         1     77:77
  DSR implemented                 1     76:76
  reserved                        2     75:74
  device size                    12     73:62            6-7
  max read current                3     61:69
  max read current                3     58-56            8
  ------------------------------------------------------------------------------
  max write current               3     55-53
  max write current               3     52-50
  device size multiplier          3     49-47            9
  erase sector size               5     46-42
  erase group size                5     41-37            10
  write protect group size        5     36-32            11
  ------------------------------------------------------------------------------
  write protect group enable      1     31:31
  manufactuter default ECC        2     30:29
  write speed factor              3     28:26
  max. write data block length    4     25:22             12
  partial blocks for write allowed 1    21:21
  reserved                         5    20:16             13
  ------------------------------------------------------------------------------
  File format group               1     15:15
  copy flag                       1     14:14
  permanent write protection      1     13:13
  temporary write protection      1     12:12
  file format                     2     11:10
  ECC code                        2      9:8              14
  --------------------------------------------------------------------------------
  CRC                             7      7:1
  Not used   always '1'           1      0:0              15
  -------------------------------------------------------------------------------
 *********************************************************************************/

MMCSD_STATUS _MMCSD_CSDCheck(uint8_t *csdbuff )
{
	uint8_t  temp;
	temp = 0;

	/*----------------------------------------------------------------------*/
	/*  CHECK FROM THE BEGINNING                                            */
	/*----------------------------------------------------------------------*/
	/*---------------------- CSD_STRUCTURE ---------------------------------*/
	temp = ( ( csdbuff[0] & 0xC0 ) >> 6 );
	if ( ( temp != 0 )&&( temp != 1 ) )
	{
		TRACE(("CSD_STRUCTURE = %x \n",temp);

		//Karen
		//return( MMCSD_CSD_MISS );//Karen modified
		//end Karen
	}
	/*------------------------- MMC_PORT -----------------------------------*/
	temp = ( ( csdbuff[0] & 0x3C ) >> 2 );
	if ( ( temp != 0 )&&( temp != 1 )&&( temp != 2 ) )
	{
		TRACE(("MMC_PORT = %x \n",temp);
		//Karen
		//        return( MMCSD_CSD_MISS );
		//end Karen
	}
	/*------------------------- TACC ---------------------------------------*/
	temp = ( csdbuff[1] & 0x80 );
	if ( temp != 0 )
	{
		TRACE(("TACC = %x \n",temp);
		//Karen
		//        return( MMCSD_CSD_MISS );
		//end Karen
	}
	/*------------------------- NSAC ---------------------------------------*/
	/*---------------------- TRAN_SPEED ------------------------------------*/
	temp = ( csdbuff[3] & 0x80 );
	if ( temp != 0 )
	{
		TRACE(("TRAN_SPEED = %x \n",temp);
		//Karen
		//        return( MMCSD_CSD_MISS );
		//end Karen
	}
	/*-------------------------- CCC ---------------------------------------*/

	//   TRACE("CCC: ----- 0x%0X\n",device->ccc);
	/*---------------------- READ_BL_LEN -----------------------------------*/
	temp = ( csdbuff[5] & 0x0F );
	if ( temp > 0x0B )
	{
		TRACE(("READ_BL_LEN = %x \n",temp);
		//Karen
		//        return( MMCSD_CSD_MISS );
		// end Karen
	}
	/*-------------------- READ_BL_PARTIAL ---------------------------------*/
	/*------------------ WRITE_BLK_MISALIGN --------------------------------*/
	/*------------------- READ_BLK_MISALIGN --------------------------------*/
	/*-------------------------- DSR_IMP -----------------------------------*/
	/*--------------------------- C_SIZE -----------------------------------*/
	/*------------------- VDD_R_CURR_MIN,VDD_W_CURR_MIN --------------------*/
	/*------------------- VDD_R_CURR_MAX,VDD_W_CURR_MAX --------------------*/
	/*------------------------- C_SIZE_MULT --------------------------------*/
	/*------------------------- SECTOR_SIZE --------------------------------*/
	/*------------------------- ERASE_GRP_SIZE -----------------------------*/
	/*-------------------------- WP_GRP_SIZE -------------------------------*/
	/*------------------------ WP_GRP_ENABLE -------------------------------*/
	/*------------------------- DEFAULT_ECC --------------------------------*/
	/*-------------------------- R2W_FACTOR --------------------------------*/
	temp = (( csdbuff[12] & 0x1C ) >> 2 );
	if ( temp > 5 )
	{

		TRACE(("R2W_FACTOR = %x \n",temp);
		//Karen
		//        return( MMCSD_CSD_MISS );
		//end Karen
	}
	/*------------------------- WRITE_BL_LEN -------------------------------*/
	/*------------------------ WRITE_BL_PARTIAL ----------------------------*/
	/*---------------------------- COPY ------------------------------------*/
	/*----------------------- PERM_WRITE_PROTECT ---------------------------*/
	/*------------------------ TMP_WRITE_PROTECT ---------------------------*/
	/*----------------------------- ECC ------------------------------------*/
	temp = ( csdbuff[14] & 0x03 );
	if ( temp > 1 )
	{

		TRACE(("ECC = %x \n",temp);
		//Karen
		//        return( MMCSD_CSD_MISS );
		//end Karen
	}
	/*----------------------------- CRC ------------------------------------*/
	temp = ( csdbuff[15] & 0x01 );
	if ( temp != 1 )
	{
		// PLAM
		return((MMCSD_STATUS)0);
		//return( MMCSD_CSD_MISS );
		// end PLAM
	}
	else{/* IF NO ERROR BEFORE HERE, DO A NORMAL RETURN                     */
		return( (MMCSD_STATUS)0 );
	}
}

/******************************************************************************
 * Function Name: _MMCSD_CIDCheck
 *
 * Input:               None.
 *
 * Value Returned:
 *
 * 		MMCSD_STATUS
 *
 * Description:
 *
 * 	Check CID register
 *
 * Modification History:
 *      30 DEC,2001, Initial version.                   Yiwei Zhao
 *****************************************************************************/

/*****************************************************************************

  CID Register:(Bit)   127  119  111  103  95  87  79   71   63  55  47  39  31  15   7    0
  ------------------------------------------------------------------------------------------
  CID Register          |MID |   OID   |        PNM               |     PSN       |MDT| CRC|
  ------------------------------------------------------------------------------------------
pCidBuff    :(BYTE)   |  0  | 1 |  2 | 4 | 5 | 7 | 8  |  9  | 10| 11| 12|13| 14 | 15 | 16|


 ******************************************************************************/

MMCSD_STATUS _MMCSD_CIDCheck( uint8_t *cidbuff )
{
	/*----------------------------------------------------------------------*/
	/*  TOTAL CHECK FROM THE BEGINNING                                      */
	/*----------------------------------------------------------------------*/
	/*-------------------------- Manufacturer ID ---------------------------*/
	if (( cidbuff[0]  == 0 )&&( cidbuff[1]  == 0 )&&( cidbuff[2]  == 0 ))
	{
		return( MMCSD_CID_MISS );
	}
	/*----------------------- Card individual number -----------------------*/
	if (( cidbuff[3]  == 0 )&&( cidbuff[4]  == 0 )&&( cidbuff[5]  == 0 )&&
			( cidbuff[6]  == 0 )&&( cidbuff[7]  == 0 )&&( cidbuff[8]  == 0 )&&
			( cidbuff[9]  == 0 )&&( cidbuff[10] == 0 )&&( cidbuff[11] == 0 )&&
			( cidbuff[12] == 0 )&&( cidbuff[13] == 0 )&&( cidbuff[14] == 0 ))
	{
		return( MMCSD_CID_MISS );
	}
	/*----------------- CRC7 checksum + not used,always'1' -----------------*/
	if ( ( cidbuff[15] & 0x01 ) != 0x01 )
	{
		// PLAM
		return 0;
		//        return( MMCSD_CID_MISS );
		// end PLAM
	}
	return( 0 );
}
/*
   MMCSD_STATUS  _MMCSD_SCRCheck(MMCSD_Dev *device)
   {
   MMCSD_Responses rp;
   MMCSD_scr scr;
   int ret;

   ret = _MMCSD_SendSCR(device,device->card_rca,&scr);
   if(ret)
   {
   FAILED("SendSCR failed\n");
   return MMCSD_ERROR;
   }

   if(scr.sd_bus_width&0x05)
   {
   INFO("can support both 1-bit & 4-bit \n");
   device->bit_width = 0;
   return MMCSD_OK;
   }
   else if(scr.sd_bus_width&0x01)
   {
   INFO("can support only 1-bit \n");
   device->bit_width = 0;
   return MMCSD_OK;
   }
   else if(scr.sd_bus_width&0x04)
   {
   INFO("can support only 4-bit \n");
   device->bit_width = 1;
   return MMCSD_OK;
   }
   else
   {
   return MMCSD_ERROR;
   }

   return MMCSD_OK;
   }
 */
