/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Copyright (C) 2002 Motorola Semiconductors HK Ltd
 *
 */


/******************************************************************************
 *
 * Copyright (C) 2001, Motorola All Rights Reserved
 *
 * File Name:   mmcsd.c
 *
 * Progammers:  Yiwei Zhao
 *
 * Date of Creations:   30  DEC,2001
 *
 * Synopsis:
 *
 * Modification History:
 * 30 DEC, 2001
 *
 *****************************************************************************/

/* Functions List:
 *
 *
 *
 */

/*
 * Implementation of the MMCSD Driver.
 */
#include <system_def.h>
#include <mx1_reg.h>
#include "mmcsd_def.h"
#include "mmcsd_main.h"

static uint32_t cur_perclk2;


#ifndef MMCSD_WITH_RTEMS
//pd25
void _mmcsd_socket1_irq_set()
{
#ifdef  CONFIG_ARCH_MX1ADS
	/*
	_reg_PTB_GIUS |= 0x00020000;
	_reg_PTB_DDIR &= ~0x00020000;
	_reg_PTB_IMR |= 0x00020000;//enable
	_reg_PTB_ICR2 |= 0x0000000C;//negative level trigger
	_reg_PTB_PUEN &= ~0x00020000;//pull up enable interrupt
	*/
	// SD presence = PB20
	_reg_PTB_GIUS |= 0x00100000;
	_reg_PTB_DDIR &= ~0x00100000;
	_reg_PTB_IMR |= 0x00100000;//enable
	_reg_PTB_ICR2 |= 0x00000300;//negative level trigger
	_reg_PTB_PUEN &= ~0x00100000;//pull up enable interrupt
#endif
#ifdef 	CONFIG_ARCH_MX2ADS
	_reg_PTD_GIUS |= 0x02000000;
	_reg_PTD_DDIR &= ~0x02000000;
	_reg_PTD_IMR  |= 0x02000000;//enable
	_reg_PTD_ICR2 |= 0x000c0000;//negative level trigger
	_reg_PTD_PUEN &= ~0x02000000;  // pull up  enable interrupt
#endif
}
void _mmcsd_socket1_clear_irq()
{
#ifdef	CONFIG_ARCH_MX1ADS
	_reg_PTB_ISR = 0x00100000;
#endif
#ifdef	CONFIG_ARCH_MX2ADS
	_reg_PTD_ISR = 0x02000000;
#endif
}
#endif /*MMCSD_WITH_RTEMS*/

/******************************************************************************
 * Function Name: _MMCSD_CheckHardwareRevision
 *
 * Input:
 *
 * Value Returned:
 *
 * Description:
 *
 *Modification History:
 *	30 DEC,2001, Initial version.                   Yiwei Zhao
 ******************************************************************************/

uint32_t	 _MMCSD_ChectHardwareRevision(const MMCSD_Dev *device)
{
	uint32_t HardwareID;

	HardwareID = mmcsd_reg_rd(device, MX1_MMCSD_REV_NO_o);
	TRACE("rev_no = %x \n",mmcsd_reg_rd(device, MX1_MMCSD_REV_NO_o));
#ifdef CONFIG_ARCH_MX1ADS
	if(HardwareID!=0x0390)
	{
		TRACE("HardwareID = 0x%x, return = %d\n", HardwareID,1);
		return 1;               //Hardware Revision Error;
	}
#endif
#ifdef CONFIG_ARCH_MX2ADS
	if(HardwareID!=0x0400)
	{
		TRACE("HardwareID = 0x%x, return = %d\n", HardwareID,1);
		return 1;               //Hardware Revision Error;
	}
#endif
	return 0;
}


/******************************************************************************
 * Function Name: _MMCSD_SoftReset
 *
 * Input:
 *
 * Value Returned:
 *
 * Description:
 *
 *Modification History:
 *	30 DEC,2001, Initial version.                   Yiwei Zhao
 ******************************************************************************/
uint32_t	 _MMCSD_SoftReset(const MMCSD_Dev *device)
{
	// System Reset
	_MMCSD_StartStopClk(device, 0x8);
	// System Reset, MMC core enable & Stop CLk
	_MMCSD_StartStopClk(device, 0xD);
	// MMC Core enable & Stop Clk
	_MMCSD_StartStopClk(device, 0x5);
	_MMCSD_StartStopClk(device, 0x5);
	_MMCSD_StartStopClk(device, 0x5);
	_MMCSD_StartStopClk(device, 0x5);
	_MMCSD_StartStopClk(device, 0x5);
	_MMCSD_StartStopClk(device, 0x5);
	_MMCSD_StartStopClk(device, 0x5);
	_MMCSD_StartStopClk(device, 0x5);

#ifdef MMCSD_DMA_ENDIAN_ERR
	// PLAM -- choose big endian FIFO mode
	mmcsd_reg_set(device, MX1_MMCSD_STR_STP_CLK_o, 0x00000020);
	// end PLAM
#else
	mmcsd_reg_clear(device, MX1_MMCSD_STR_STP_CLK_o, 0x00000020);
#endif
	//Set MMC Response Time-Out Register
	mmcsd_reg_wr(device, MX1_MMCSD_RES_TO_o, 0xffff);

	//Set Block length register
	mmcsd_reg_wr(device, MX1_MMCSD_BLK_LEN_o, MMCSD_MINIMUM_BLK_SIZE); //2002/04/19

	//Set MMC Number of Blocks Register
	mmcsd_reg_wr(device, MX1_MMCSD_NOB_o, 1); //2002/04/19

	return 0;
}

void _MMCSD_ReadTimeOut(const MMCSD_Dev *device, uint32_t arg)
{
	arg = 0x2db4;
	mmcsd_reg_wr(device, MX1_MMCSD_READ_TO_o, 0x2db4);
}

/******************************************************************************
 * Function Name: _MMCSD_SetCmd
 *
 * Input:
 *
 * Value Returned:
 *
 * Description:
 *
 *Modification History:
 *	30 DEC,2001, Initial version.                   Yiwei Zhao
 ******************************************************************************/
void _MMCSD_SetCmd(const MMCSD_Dev *device, uint8_t cmd,uint32_t arg,uint32_t ctrl)
{
#ifdef MMCSD_CMD
	TRACE("	----------------------------------------\n");
	TRACE("	CMD%02d  Arg: 0x%08lX  Ctrl: 0x%04lX\n",
			cmd,(unsigned long)arg,(unsigned long)ctrl);
#endif
	mmcsd_reg_wr(device, MX1_MMCSD_CMD_o, cmd);
	mmcsd_reg_wr(device, MX1_MMCSD_ARGH_o, arg>>16);
	mmcsd_reg_wr(device, MX1_MMCSD_ARGL_o, arg&0xFFFF);
	mmcsd_reg_wr(device, MX1_MMCSD_CMD_DAT_CONT_o, ctrl);
}

/******************************************************************************
 * Function Name: _MMCSD_IsRun
 *
 * Input:
 *
 * Value Returned:
 *
 * Description:
 *
 *Modification History:
 *	30 DEC,2001, Initial version.                   Yiwei Zhao
 ******************************************************************************/
uint32_t _MMCSD_IsRun(const MMCSD_Dev *device)
{
	return mmcsd_reg_rd(device, MX1_MMCSD_STATUS_o)&0x100;
}

/******************************************************************************
 * Function Name: _MMCSD_StartClk
 *
 * Input:
 *
 * Value Returned:
 *
 * Description:
 *
 *Modification History:
 *	30 DEC,2001, Initial version.                   Yiwei Zhao
 ******************************************************************************/
void _MMCSD_StartClk(const MMCSD_Dev *device)
{
	unsigned short count = 0;
	do
	{
		if (count++ == 0)
		{
			// do this every 256 loops
			mmcsd_reg_set(device, MX1_MMCSD_STR_STP_CLK_o, 0x06);
		}
	}
	while(!_MMCSD_IsRun(device));
}

/******************************************************************************
 * Function Name: _MMCSD_StopClk
 *
 * Input:
 *
 * Value Returned:
 *
 * Description:
 *
 *Modification History:
 *	30 DEC,2001, Initial version.                   Yiwei Zhao
 ******************************************************************************/
void _MMCSD_StopClk(const MMCSD_Dev *device)
{
	unsigned short count = 0;
	do
	{
		if (count++ == 0)
		{
			// do this every 256 loops
			mmcsd_reg_set(device, MX1_MMCSD_STR_STP_CLK_o, 0x05);
		}
	}while(_MMCSD_IsRun(device));
}

/******************************************************************************
 * Function Name: _MMCSD_GetResponses
 *
 * Input:
 *
 * Value Returned:
 *
 * Description:
 *
 *Modification History:
 *	30 DEC,2001, Initial version.                   Yiwei Zhao
 ******************************************************************************/

void _MMCSD_GetResponses(const MMCSD_Dev *device, MMCSD_Responses *pRS, uint8_t  rsType)
{
	int i;
    	uint32_t temp = 0;

	if(rsType==MMCSDB_R1)
	{
		pRS->status=0;
		temp=mmcsd_reg_rd(device, MX1_MMCSD_RES_FIFO_o);

		pRS->status|=(temp&0x00FF)<<24;

		temp=mmcsd_reg_rd(device, MX1_MMCSD_RES_FIFO_o);
		pRS->status|=temp<<8;

		temp=mmcsd_reg_rd(device, MX1_MMCSD_RES_FIFO_o);
		pRS->status|=(temp&0xFF00)>>8;

	}
	else if(rsType==MMCSDB_R3)
	{
		pRS->OCR=0;

		temp=mmcsd_reg_rd(device, MX1_MMCSD_RES_FIFO_o);
		pRS->OCR|=(temp&0x00FF)<<24;

		temp=mmcsd_reg_rd(device, MX1_MMCSD_RES_FIFO_o);
		pRS->OCR|=temp<<8;

		temp=mmcsd_reg_rd(device, MX1_MMCSD_RES_FIFO_o);
		pRS->OCR|=(temp&0xFF00)>>8;
	}
	else
	{
		for(i=0;i<16;)
		{
			temp=(uint32_t)mmcsd_reg_rd(device, MX1_MMCSD_RES_FIFO_o);
			pRS->CSD[i] = (temp&0xFF00)>>8;
			i++;
			pRS->CSD[i] = (temp&0x00FF);
			i++;
		}
	}

}

/******************************************************************************
 * Function Name: _MMCSD_SetBlockLenReg
 *
 * Input:
 * 	 size:		size of the block.
 *
 * Value Returned:
 *
 *
 * Description:
 * 	this routine sets the size of a block
 *
 *Modification History:
 *	30 DEC,2001, Initial version.                   Yiwei Zhao
 ******************************************************************************/
void _MMCSD_SetBlockLenReg(const MMCSD_Dev *device, uint16_t size)
{
	mmcsd_reg_wr(device, MX1_MMCSD_BLK_LEN_o, size);
}

/******************************************************************************
 * Function Name: _MMCSD_SetNumberOfBlockReg
 *
 * Input:
 *
 * 	num:	Number of block
 *
 * Value Returned:
 *
 *
 * Description:
 * 	this routine sets number of the blocks.
 *
 *Modification History:
 *	30 DEC,2001, Initial version.                   Yiwei Zhao
 ******************************************************************************/

void _MMCSD_SetNumberOfBlockReg(const MMCSD_Dev *device, uint16_t num)
{
	mmcsd_reg_wr(device, MX1_MMCSD_NOB_o, num);
}

/******************************************************************************
 * Function Name: _MMCSD_ClockSet
 *
 * Input:
 *
 * 	speed:		if speed == oxff, set the highest speed.
 *
 * Value Returned:
 *
 *
 * Description:
 * 	this routine set clock rate.
 *
 *Modification History:
 *	30 DEC,2001, Initial version.                   Yiwei Zhao
 ******************************************************************************/

void _MMCSD_ClockSet(const MMCSD_Dev *device, uint32_t prescaler,uint8_t clk)
{
	_MMCSD_StopClk(device);

#ifdef CONFIG_ARCH_MX1ADS
	prescaler = prescaler&0x07;
	clk = clk&0x07;
	mmcsd_reg_wr(device, MX1_MMCSD_CLK_RATE_o, (prescaler<<3)|clk);
#endif
#ifdef CONFIG_ARCH_MX2ADS
	prescaler = prescaler&0x0f;
	clk = clk&0xfff;
	mmcsd_reg_wr(device, MX1_MMCSD_CLK_RATE_o, (prescaler<<4)|clk);
#endif
}

void _MMCSD_SetClk(const MMCSD_Dev *device, unsigned freq)
{
	unsigned prescaler;
	unsigned clkrate=20000000;	/* expected to be 20 MHz */
	unsigned clkdiv;

	if(freq > MMCSD_MAX_CLKRATE)
		freq = MMCSD_MAX_CLKRATE;

#ifdef CONFIG_ARCH_MX1ADS
        if(freq != MMCSD_LOW_CLKRATE)
	{
		prescaler=(cur_perclk2+(clkrate*7)/8)/clkrate;
		clkrate=cur_perclk2/prescaler;
		switch(prescaler) {
			case 0:
			case 1: prescaler = 0;
        			break;
			case 2: prescaler = 1;
        			break;
			case 3: prescaler = 2;
        			break;
			case 4: prescaler = 4;
        			break;
			default:
			case 5: prescaler = 5;
        			break;
		}

		for(clkdiv=0; clkdiv<7; clkdiv++) {
			if(clkrate / (1<<clkdiv) <= freq)
                                break;
                }

		_MMCSD_ClockSet(device, prescaler,clkdiv);
		TRACE("setting high speed clkrate=0x%x ,perclk2=%d Hz\n",
			mmcsd_reg_rd(device, MX1_MMCSD_CLK_RATE_o),cur_perclk2);
	}
	else
	{
		_MMCSD_ClockSet(device, 7,7);
		TRACE("setting low speed clkrate=0x%x ,perclk2=%d Hz\n",
			mmcsd_reg_rd(device, MX1_MMCSD_CLK_RATE_o),cur_perclk2);
	}
#endif
#ifdef CONFIG_ARCH_MX2ADS
	if(freq != MMCSD_LOW_CLKRATE)
	{
		if(cur_perclk2<20000000)
		{
			_MMCSD_ClockSet(device, 0,1);
		}
		else
		{
			_MMCSD_ClockSet(device, 0,cur_perclk2/20000000+1);
		}
	}
	else
	{
		_MMCSD_ClockSet(device, 8,0xa);//if perclk2 is about 20M
	}
        TRACE("clkrate=0x%x ,perclk2=%d Hz\n",mmcsd_reg_rd(device, MX1_MMCSD_CLK_RATE_o),cur_perclk2);
#endif
}
/******************************************************************************
 * Function Name: _MMCSD_PinConfig
 *
 * Input:
 *
 * Value Returned:
 *
 * Description:
 * 	this routine sets GPIO registers.
 *
 *Modification History:
 *	30 DEC,2001, Initial version.                   Yiwei Zhao
 ******************************************************************************/

static void _MMCSD_get_cur_perclk2(void)
{
	/*FIXME: this function does not belong to MMCSD code*/
#ifdef CONFIG_ARCH_MX1ADS
	cur_perclk2 = _reg_PLL_PCDR;
	cur_perclk2 = (cur_perclk2&0x000000F0)>>4;
	cur_perclk2 = 96000000/(cur_perclk2+1);

#endif
#ifdef CONFIG_ARCH_MX2ADS
	cur_perclk2 = _reg_PLL_PCDR;
	cur_perclk2 = (cur_perclk2&0x03c00000)>>22;
	cur_perclk2 = 128000000/(cur_perclk2+1);

#endif
}
void _MMCSD_PinConfig(void)
{
	_MMCSD_get_cur_perclk2();
#ifdef CONFIG_ARCH_MX1ADS
	_reg_PTB_GIUS &= 0xFFFFC0FF;
	_reg_PTB_GPR &= 0xFFFFC0FF;
	_reg_PTB_PUEN |= 0x00003f00;
#endif

#ifdef CONFIG_ARCH_MX2ADS
	_reg_PTE_DDIR |= 0x00fc0000;//0x3f000000;
	_reg_PTE_GIUS &= 0xff03ffff;
	_reg_PTE_GPR  &= 0xff03ffff;
	_reg_PTE_PUEN |= 0x00fc0000;

	//add for SD2 pb4~pb9 pb7-dat3,pb9-clk
	_reg_PTB_DDIR |= 0x000003f0;
	_reg_PTB_GIUS &= ~0x000003f0;
	_reg_PTB_GPR  &= ~0x000003f0;
	_reg_PTB_PUEN |= 0x000003f0;

#endif
}
void _MMCSD_64M_MMC_PinConfig(void)
{
	_MMCSD_get_cur_perclk2();
#ifdef CONFIG_ARCH_MX2ADS
	//pin pe18~23,18,19,20,22 up, 21-dat3,23-clk
	_reg_PTE_DDIR(GPIOE) |= 0x00fc0000;
	_reg_PTE_GIUS(GPIOE) &= 0xff03ffff;
	_reg_PTE_GPR(GPIOE)  &= 0xff03ffff;
	_reg_PTE_PUEN(GPIOE) |= 0x005c0000;
//	_reg_PTE_PUEN(GPIOE) &= 0xff5fffff;

	//add for SD2 pb4~pb9 pb7-dat3,pb9-clk
	_reg_PTB_DDIR |= 0x000003f0;
	_reg_PTB_GIUS &= ~0x000003f0;
	_reg_PTB_GPR  &= ~0x000003f0;
	_reg_PTB_PUEN |= 0x00000170;
	_reg_PTB_PUEN &= 0xfffffd7f;


#endif
	//pb8~13, 11-dat3,12-clk,13-cmd
#ifdef CONFIG_ARCH_MX1ADS
	_reg_PTB_DDIR |= 0x00003F00;		// Enable SDHC's port to Output
	_reg_PTB_GIUS &= 0xFFFFC0FF;
	_reg_PTB_GPR  &= 0xFFFFC0FF;
	_reg_PTB_PUEN |= 0x00003700;

	_reg_PTB_PUEN  &=0xfffff7ff;		//Karen modify it for MMC test

	_reg_PTB_PUEN |= 0x00003F00;
#endif
}
/************************************************

put content to STR_STP_CLK
 uint32_t Clk_en = set the clk to start or stop
	0x6: start MMC clk
	0x5: stop MMC clk

************************************************/
void _MMCSD_StartStopClk(const MMCSD_Dev *device, uint32_t Clk_en)
{
    	//when the clk is STARTED check whether the MMC is enable and reset is not set
	if(((Clk_en&0x2)!=0)&&((Clk_en&0x8)==0)&&((Clk_en&0x4)!=0))
	{
		mmcsd_reg_clear(device, MX1_MMCSD_STR_STP_CLK_o, MMCSD_CLOCK_MASK);
		mmcsd_reg_set(device, MX1_MMCSD_STR_STP_CLK_o, Clk_en);
		while ( (mmcsd_reg_rd(device, MX1_MMCSD_STATUS_o)&0x100) == 0 );//check the status if the clock do not start
	//when the clk is STOPPED, check whether the MMC is enable and reset is not set
	}
	else if(((Clk_en&0x1)!=0)&&((Clk_en&0x8)==0)&&((Clk_en&0x4)!=0))
	{
		mmcsd_reg_clear(device, MX1_MMCSD_STR_STP_CLK_o, MMCSD_CLOCK_MASK);
		mmcsd_reg_set(device, MX1_MMCSD_STR_STP_CLK_o, Clk_en);
		while ( (mmcsd_reg_rd(device, MX1_MMCSD_STATUS_o) & 0x100) != 0 );//check the status if the clock is still running
	}
	//reset or enable the syste
	else
	{
		mmcsd_reg_clear(device, MX1_MMCSD_STR_STP_CLK_o, MMCSD_CLOCK_MASK);
		mmcsd_reg_set(device, MX1_MMCSD_STR_STP_CLK_o, Clk_en);
	}
}

