/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Copyright (C) 2002 Motorola Semiconductors HK Ltd
 *
 */

/******************************************************************************
 *
 * Copyright (C) 2001, Motorola All Rights Reserved
 *
 * File Name:   mmcsd_main.h
 *
 * Progammers: Yiwei Zhao
 *
 * Date of Creations:   30 Nov 2001
 *
 * Synopsis:
 *
 * Modification History:
 * 30 Nov, 2001, initialization version
 *
 *****************************************************************************/

/*
 * Implementation of the MultiMediaCard/SD Driver.
 */
#ifndef _MOT_MMC_SD_MAIN_H_
#define _MOT_MMC_SD_MAIN_H_
#include <system_def.h>

#ifdef CONFIG_ARCH_MX2ADS
#include <mx2.h>
#include <dma.h>
#endif
#include "sys_mmcsd.h"
#include <rtems.h>
#include <rtems/blkdev.h>

#ifdef SILICON_MASK_0L
// only needed if 0L44N/0L45N is used.  Add -DMASK_SILICON_0L to Makefile if using 0L44N/0L45N
#define MMCSD_DMA_ENDIAN_ERR 	1
#endif

#define MMCSD_WITH_RTEMS	1
#define MMCSD_TEST	1
//#define MMCSD_WRITE_DMA	1
//#define MMCSD_READ_DMA	1
#define MMCSD_CMD	1

#define DBMX1_DEBUG 1
#ifdef DBMX1_DEBUG
#define TRACE(fmt, args...) \
	{ \
		printk("%s:%d:%s:\n",__FILE__, __LINE__,__FUNCTION__); \
		printk(fmt, ## args);\
	}
#else
#define TRACE(fmt, args...)
#endif

#define FAILED(fmt, args...) \
	{ \
		printk("%s:%d:%s:\n",__FILE__, __LINE__,__FUNCTION__); \
		printk(fmt, ## args);\
	}

#define INFO(fmt, args...) \
	{ \
		printk(fmt, ## args);\
	}


/*****************************************************************************
 *
 * 			Macro Definition
 *
 ****************************************************************************/

/* 3.2.	 Constant Definition
 * 3.2.1.Addresses of Registers
 * The following macros define the addresses of SDHC's registers.
 */

/* 3.2.2 Card State
 *The macros define the states of MMC/SD card.
 */
#define    MMCSD_NOTREADY	0x0	/*Initial State*/
#define    MMCSD_INACTIVE	0x1	/*Inactive State*/


#define    MMCSD_IDLE				0x0	/*Idle State*/
#define    MMCSD_READY				0x200	/*Ready State */
#define    MMCSD_IDENTIFICATION		0x400	/*Identification State */
#define    MMCSD_STANDBY			0x600	/*Stand-by State*/
#define    MMCSD_TRANSFER			0x800	/*Transfer State*/
#define    MMCSD_SEND_DATA			0xA00	/*Sending State*/
#define    MMCSD_RECEIVE_DATA		0xC00	/*Receiving State*/
#define    MMCSD_PROGRAM			0xE00	/*Programming State*/
#define    MMCSD_DISCONNECT			0x1000	/* Disconnect State*/
#define	   MMCSD_STATUS_STATE		0x00001E00

/* 3.2.3. Read Ahead Value
 * This macro defines the block-level read-ahead values for MMCSD driver.
 */
#define 	MMCSD_READ_AHEAD	8

/* 3.2.4. Device Number
 * This macro defines the number of devices that MS driver will manage.
 */
#define 	MMCSD_DEVICE_NUM	2

/* 3.2.5. Block and Sector Size
 * These macros define the sector size and minimum block size, which are
 * required by Linux Block Driver.
 */
#define 	MMCSD_MINIMUM_BLK_SIZE		512
#define 	MMCSD_FS_SECTOR_SIZE		MMCSD_MINIMUM_BLK_SIZE
#define		MMCSD_MINIMUM_BLK_SIZE_BIT	9
#define		MMCSD_MAX_MULTI_BLK			64

/* MMCSD Major Numner
 *
 */

/* there is no problem to use dynamic device numbers under RTEMS */
/*#define 	MMCSD_MAJOR			0x78*/

/* MMCSD Interrupr Number
 *
 */
#ifdef CONFIG_ARCH_MX2ADS
#define 	MMCSD_IRQ			11
#define		MMCSD2_IRQ			10
#define		MMCSD_SOCKET1_INT	8	//PD25
#endif
#ifdef CONFIG_ARCH_MX1ADS
#define 	MMCSD_IRQ			35
#define		MMCSD_SOCKET1_INT	12	//PB20
#endif
/* MMCSD DMA Channel
 *
 */
#ifdef CONFIG_ARCH_MX2ADS
#define 	MMCSD_DMA_CHANNEL		0x7
#endif
#ifdef CONFIG_ARCH_MX1ADS
#define 	MMCSD_DMA_CHANNEL		0xD
#endif
#define		MMCSD_READ			0x1
#define		MMCSD_WRITE			0x2

/*
 * For Setting Start Stop Speed
 */
/*FIXME: we need to limit clkrate under 20MHz to achieve stable operation of SD cards */
#define MMCSD_MAX_CLKRATE  	20000000/2	// max 20 MHz
#define MMCSD_LOW_CLKRATE	0		// As Low as possible





/*****************************************************************************
 *
 * 		Structure Definition
 *
 ****************************************************************************/

/* 3.3.	Structure Definition
 * 3.3.1. Card Status
 * Structure of MMCSD Card Status Register
 */
/*	typedef  struct _CARD_STATUS  {

		uint32_t	res1			:5;
		uint32_t	app_cmd			:1;
		uint32_t	res2			:2;
		uint32_t	ready_for_data		:1;
		uint32_t	cur_state		:4;
		uint32_t	erase_reset		:1;
		uint32_t	card_ecc_disabled	:1;
		uint32_t	wp_erase_skip		:1;
		uint32_t	cidcsd_overwrite	:1;
		uint32_t	overrun			:1;
		uint32_t	underrun		:1;
		uint32_t	error			:1;
		uint32_t	cc_error		:1;
		uint32_t	card_ecc_failed		:1;
		uint32_t	illegal_cmd		:1;
		uint32_t	com_crc_error		:1;
		uint32_t	lock_unlock_failed	:1;
		uint32_t	card_is_locked		:1;
		uint32_t	wp_violation		:1;
		uint32_t	erase_param		:1;
		uint32_t	erase_seq_error		:1;
		uint32_t	block_len_error		:1;
		uint32_t	address_error		:1;
		uint32_t	out_of_range		:1;

	} MMCSD_CARD_STATUS;
*/

/* 3.3.2. SDHC Status Register
 * Structure of SDHC Status Register
 */
/*
	typedef  struct _SDHC_STAT  {

		uint32_t	timeout_read		:1;
		uint32_t	timeout_reponse		:1;
		uint32_t	crc_werror		:1;
		uint32_t	crc_rerror		:1;
		uint32_t	res1			:1;
		uint32_t	crc_resperror		:1;
		uint32_t	app_bfe			:1;
		uint32_t	app_bff			:1;
		uint32_t	clock_running		:1;
		uint32_t	wr_crc_error		:2;
		uint32_t	data_tran_done		:1;
		uint32_t	access_op_done		:1;
		uint32_t	end_cmd_resp		:1;
		uint32_t	irq_active		:1;
		uint32_t	card_presence		:1;
		uint32_t	res2			:16;

	} DBMX1_SDHC_STATUS;
*/
/* 3.3.3 SDHC Clock Rate Register
 */

	typedef  struct _SDHC_CLK_RATE  {

		uint32_t	clk_rate 		:3;
		uint32_t	pre_scaler		:3;
		uint32_t	res1			:26;

	} DBMX1_SDHC_CLK_RATE;

/* 3.3.4 Command and Data Control Register
 */

	typedef  struct _SDHC_CMD_DAT_CONT {

		uint32_t	resp_format		:3;
		uint32_t	data_enable		:1;
		uint32_t	write_read		:1;
		uint32_t	stream_block		:1;
		uint32_t	busy			:1;
		uint32_t	initialize		:1;
		uint32_t	bus_width		:2;
		uint32_t	start_read_wait		:1;
		uint32_t	stop_read_wait		:1;
		uint32_t	cmdres			:1;
		uint32_t	res			:19;

	} DBMX1_SDHC_CMD_DAT_CONT;

/* 3.3.5 Interrupt Mask Register
 */
/*
	typedef  struct _SDHC_INT_MASK  {

		uint32_t	data_tran		:1;
		uint32_t	prg_done		:1;
		uint32_t	end_cmd_res		:1;
		uint32_t	buf_ready		:1;
		uint32_t	sdio_interrupt		:1;
		uint32_t	dat0_irq		:1;
		uint32_t	auto_card		:1;
		uint32_t	res			:25;

	} DBMX1_SDHC_INT_MASK;
*/
/* 3.3.6 MMCSD Device Structure
 *
 */
	typedef struct _MMCSD_Dev {
		uint32_t	base_addr;	// the respective SDHC base address
		uint8_t		CID[16];
		uint32_t	card_rca;	// card address
		uint32_t	card_state;	// current state of the card

		uint32_t	dev_size;
		uint32_t	sect_num;
		uint8_t		blksize_bit;
	   	uint32_t	blksize;	// size of block in file system

		uint16_t	phy_blksize;	// Physical size of block on the MMCSD.
		uint32_t	phy_sectorsize;	// the size of erasable sector
		uint32_t	wp_grp_size;	// the size of write protected group

	   	rtems_id  	lock;
		rtems_id	sema;
	   	uint32_t	usage;

//	   	request_queue_t	queue;		// request queue
		uint8_t		busy;
		int		kpid;

		volatile uint8_t  pending_irq;	// There has been registered IRQ
		volatile rtems_id select_wait;	// Kernel thread blocked on it.
//		rtems_id	dma_wait;	// Kernel thread blocked on it.
		volatile uint8_t  cur_cmd;	// What command being executed.
		volatile uint32_t result;	// set by tasklet
		uint8_t	*buff;

		uint8_t		card_type;
		uint8_t		bit_width;	//1: 4-bit, 0: 1-bit
		uint32_t	lba_sec_offset;

		uint8_t	*buff_pool[MMCSD_MAX_MULTI_BLK];
#ifdef MMCSD_DMA_ENDIAN_ERR
		uint8_t	*tmp_pool[MMCSD_MAX_MULTI_BLK];
#endif
		uint32_t	cur_num;
		uint32_t	cur_sec;
		uint32_t	addr;

		int		irq_number;
		uint32_t	dma_channel;
#ifdef 	CONFIG_ARCH_MX2ADS
		dma_request_t	dma;
#endif
		uint32_t	is_ro;
		uint32_t	port;		// in MX21 there are two ports
		uint16_t	ccc;		// Card Command Class
		uint32_t	max_dtr;	// maximal data transfer rate
	} MMCSD_Dev;

/* The Lock for Current Selected Device
 *
 */

typedef struct _selected_lock{

	   	rtems_id 	lock;
		MMCSD_Dev	*device;
}MMCSD_Lock;

/* 3.3.7 MMCSD Blocks
 *
 */
	typedef struct _MMCSD_Blks {

		uint32_t	start;		// start address

	}MMCSD_Blks;

/* 3.3.8 MMCSD Password Structure
 *
 */
	typedef struct _MMCSD_PWD {

		uint8_t	set_pwd		:1;	// new password
		uint8_t	clr_pwd		:1;	// clear password
		uint8_t	lock		:1;	// lock, 0 unlock
		uint8_t	erase		:1;	// Forced Erase
		uint8_t	res		:4;
	   	uint8_t	len;			// length of password
		uint8_t   pwd[256];

	}MMCSD_PWD;


typedef union _Responses {

		uint32_t	status;
		uint32_t	OCR;
		uint8_t	CID[16];
		uint8_t	CSD[16];
		uint8_t	SCR[8];
}MMCSD_Responses;

typedef struct tag_csd{

	uint8_t    csd_structure;       /* CSD structure              */
	uint8_t    mmc_prot;            /* MMC protocol version       */
	uint16_t   read_bl_len;         /* read data block length     */
	uint32_t    c_size;              /* device size                */
	uint16_t	sector_size;         /* erase sector size          */
}MMC_CSD;

/*****************************************************************************
 *
 * 		Extern Variables
 *
 ****************************************************************************/

extern MMCSD_Dev	*g_mmcsd_devices[MMCSD_DEVICE_NUM];
extern uint8_t		g_mmcsd_num;
extern MMCSD_Lock	g_selected_lock[MMCSD_DEVICE_NUM];
extern uint32_t		g_mmcsd_irq_count;

/*****************************************************************************
 *
 * 		Function Declarations
 *
 ****************************************************************************/
/* Functions in mmcsd_drv.c*/

MMCSD_STATUS _MMCSD_InitCard(void);
MMCSD_STATUS _MMCSD_InitMMC(MMCSD_Dev *device);
MMCSD_STATUS _MMCSD_GetInfo(MMCSD_Dev *device);
MMCSD_STATUS _MMCSD_SetSectLen(MMCSD_Dev *device);
uint32_t _MMCSD_GetStatus(MMCSD_Dev *device);
void _MMCSD_SetCSD(MMCSD_Dev *device,uint8_t * Csd_buff);
MMCSD_STATUS _MMCSD_CSDCheck(uint8_t *csdbuff );
MMCSD_STATUS _MMCSD_CIDCheck( uint8_t *cidbuff );
void _MMCSD_SetCID(MMCSD_Dev *device,uint8_t* Cid_buff);
MMCSD_STATUS _MMCSD_InitSDHC(const MMCSD_Dev *device);
//MMCSD_STATUS _MMCSD_SCRCheck(MMCSD_Dev *device);

/* Functions in mmcsd_cmd.c*/

MMCSD_STATUS 	_MMCSD_GoIdleState(MMCSD_Dev *device);
MMCSD_STATUS 	_MMCSD_SendOpCond(MMCSD_Dev *device, uint32_t voltage);

MMCSD_STATUS 	_MMCSD_AllSendCID(MMCSD_Dev *device, uint8_t *pCidBuff);
MMCSD_STATUS 	_MMCSD_SetRelativeAddr(MMCSD_Dev *device, uint32_t *addr);
MMCSD_STATUS 	_MMCSD_SelectDeselectCard(MMCSD_Dev *device, uint32_t addr);
MMCSD_STATUS 	_MMCSD_SendCSD(MMCSD_Dev *device, uint32_t addr,uint8_t* pBuff);
MMCSD_STATUS 	_MMCSD_SendCID(MMCSD_Dev *device, uint32_t addr,uint8_t * pBuff);
MMCSD_STATUS  	_MMCSD_SendStatus(MMCSD_Dev *device, uint32_t addr,uint32_t *pStatus);
MMCSD_STATUS 	_MMCSD_SetBlockLen(MMCSD_Dev *device, uint32_t size,uint32_t num);
MMCSD_STATUS 	_MMCSD_TagSectorStart(MMCSD_Dev *device, uint32_t addr);
MMCSD_STATUS 	_MMCSD_TagSectorEnd(MMCSD_Dev *device, uint32_t addr);
MMCSD_STATUS 	_MMCSD_Erase(MMCSD_Dev *device);
MMCSD_STATUS 	_MMCSD_NoDataCommand(MMCSD_Dev *device, uint8_t cmd, uint32_t arg,uint32_t ctrl);
MMCSD_STATUS 	_MMCSD_WaitUntil(MMCSD_Dev *device, uint32_t EndCondition);
MMCSD_STATUS    _MMCSD_SetWriteProt(MMCSD_Dev *device, uint32_t addr);
MMCSD_STATUS 	_MMCSD_SetBusWidth(MMCSD_Dev *device, uint32_t rca, uint32_t width);
void 		_MMCSD_Mask_interrupt(const MMCSD_Dev *device, uint32_t mask, uint32_t *orig);
void 		_MMCSD_Restore_Mask(const MMCSD_Dev *device, uint32_t orig);
MMCSD_STATUS 	_MMCSD_irq_WaitUntil(MMCSD_Dev *device, uint32_t EndCondition,uint8_t cmd,int timeout);
//MMCSD_STATUS	_MMCSD_SendSCR(const MMCSD_Dev *device, uint32_t rca,MMCSD_scr *scr);
MMCSD_STATUS	_MMCSD_ClearWriteProt(MMCSD_Dev *device, uint32_t addr);


/* Functions in mmcsd_io.c */

uint32_t	 	_MMCSD_ChectHardwareRevision(const MMCSD_Dev *device);
uint32_t		_MMCSD_SoftReset(const MMCSD_Dev *device);
void 	_MMCSD_SetCmd(const MMCSD_Dev *device, uint8_t cmd,uint32_t arg,uint32_t ctrl);
uint32_t 	_MMCSD_IsRun(const MMCSD_Dev *device);
void 	_MMCSD_StartClk(const MMCSD_Dev *device);
void 	_MMCSD_StopClk(const MMCSD_Dev *device);
void 	_MMCSD_GetResponses(const MMCSD_Dev *device, MMCSD_Responses *pRS, uint8_t  rsType);
void 	_MMCSD_SetBlockLenReg(const MMCSD_Dev *device, uint16_t size);
void 	_MMCSD_SetNumberOfBlockReg(const MMCSD_Dev *device, uint16_t num);
void 	_MMCSD_StartStopClk(const MMCSD_Dev *device, uint32_t Clk_en);
void 	_MMCSD_ClockSet(const MMCSD_Dev *device, uint32_t prescaler,uint8_t clk);
void  	_MMCSD_ReadTimeOut(const MMCSD_Dev *device, uint32_t arg);
void 	_MMCSD_64M_MMC_PinConfig(void);
void	_MMCSD_SetClk(const MMCSD_Dev *device, unsigned freq);

void _mmcsd_socket1_clear_irq(void);
void _mmcsd_socket1_irq_set(void);

/* Functions for Testing */

void _mmcsd_test_reading_pages(MMCSD_Dev * device);
void _mmcsd_print_page(uint8_t * p, uint32_t size);

#define MMCSD_OK	0
#define MMCSD_ERROR	-1

/* Exported API functions */

int mmcsd_init(void);
int mmcsd_open_device(MMCSD_Dev *device);
int mmcsd_release_device(MMCSD_Dev *device);
int mmcsd_revalidate_device(MMCSD_Dev *device);
int mmcsd_ioctl_device(MMCSD_Dev *device, unsigned command, unsigned long  argument);
int mmcsd_read_device(MMCSD_Dev *device, char * buffer, uint32_t sector, uint32_t nr_sector);
int mmcsd_write_device(MMCSD_Dev *device, char * buffer, uint32_t sector, uint32_t nr_sector);

/* RTEMS specific functions and code */

/* The SBWAIT_EVENT is reused there, it is used by select and cannot be freely not be used by application code */
#define MMCSD_WAIT_EVENT   RTEMS_EVENT_24
#define MMCSD_INNVALID_TID 0

int mmcsd_device_lock_init(MMCSD_Dev *device);
int mmcsd_device_lock(MMCSD_Dev *device);
int mmcsd_device_unlock(MMCSD_Dev *device);

int mmcsd_device_sem_init(MMCSD_Dev *device);
int mmcsd_device_sem_obtain(MMCSD_Dev *device);
int mmcsd_device_sem_release(MMCSD_Dev *device);

int mmcsd_device_select_init(MMCSD_Dev *device);
long mmcsd_device_select_wait(MMCSD_Dev *device, long timeout);
int mmcsd_device_select_wake(MMCSD_Dev *device);

static inline uint16_t mmcsd_reg_rd(const MMCSD_Dev *device, unsigned reg_offs)
{
  return *(volatile uint32_t*)(device->base_addr+reg_offs);
}

static inline void mmcsd_reg_wr(const MMCSD_Dev *device, unsigned reg_offs, uint16_t val)
{
  *(volatile uint32_t*)(device->base_addr+reg_offs)=val;
}

static inline void mmcsd_reg_set(const MMCSD_Dev *device, unsigned reg_offs, uint16_t val)
{
  *(volatile uint32_t*)(device->base_addr+reg_offs)|=val;
}

static inline void mmcsd_reg_clear(const MMCSD_Dev *device, unsigned reg_offs, uint16_t val)
{
  *(volatile uint32_t*)(device->base_addr+reg_offs)&=~val;
}

#endif


